@@ -82,7 +82,10 @@ namespace GmmLib
 #if(!defined(__GMM_KMD__) && !GMM_LIB_DLL_MA)
     	static int32_t                   RefCount;
 #endif
-        GMM_CLIENT                       ClientType;
+#if GMM_LIB_DLL_MA
+        int32_t                          RefCount;
+#endif //GMM_LIB_DLL_MA
+	GMM_CLIENT                       ClientType;
         GMM_PLATFORM_INFO_CLASS*         pPlatformInfo;
 
         GMM_TEXTURE_CALC*                pTextureCalc;
@@ -119,11 +122,20 @@ namespace GmmLib
         static GMM_MUTEX_HANDLE           SingletonContextSyncMutex;
 #endif
         GMM_PRIVATE_PAT PrivatePATTable[GMM_NUM_PAT_ENTRIES];
+        GMM_MUTEX_HANDLE SyncMutex;         // SyncMutex to protect access of Gmm UMD Lib process Singleton Context
     public :
         //Constructors and destructors
         Context();
         ~Context();
 
+        GMM_STATUS GMM_STDCALL          LockSingletonContextSyncMutex();
+        GMM_STATUS GMM_STDCALL          UnlockSingletonContextSyncMutex();
+
+#if GMM_LIB_DLL_MA
+        int32_t                         IncrementRefCount();
+        int32_t                         DecrementRefCount();
+#endif //GMM_LIB_DLL_MA
+
 #if(!defined(__GMM_KMD__) && (!GMM_LIB_DLL_MA))
         static int32_t IncrementRefCount()  // Returns the current RefCount and then increment it
         {
@@ -311,7 +323,7 @@ namespace GmmLib
         }
 
     #ifdef GMM_LIB_DLL
-        ADAPTER_BDF             sBdf;
+        ADAPTER_BDF             sBdf;  // Adpater's Bus, Device and Function info for which Gmm UMD Lib process Singleton Context is created
         #ifdef _WIN32
             // ProcessHeapVA Singleton HeapObj
             GMM_HEAP            *pHeapObj;
@@ -351,7 +363,7 @@ namespace GmmLib
                     SingletonContextSyncMutex = NULL;
                 }
             }
-
+            #if !GMM_LIB_DLL_MA
             /////////////////////////////////////////////////////////////////////////
             /// Acquire Sync Mutex for Singleton Context access
             /////////////////////////////////////////////////////////////////////////
@@ -383,6 +395,7 @@ namespace GmmLib
                     return GMM_ERROR;
                 }
             }
+            #endif //!GMM_LIB_DLL_MA
 
         #else // Non Win OS
 
@@ -393,7 +406,7 @@ namespace GmmLib
             {
                 pthread_mutex_destroy(&SingletonContextSyncMutex);
             }
-
+            #if !GMM_LIB_DLL_MA
             /////////////////////////////////////////////////////////////////////////
             /// Acquire Sync Mutex for Singleton Context access
             /////////////////////////////////////////////////////////////////////////
@@ -411,6 +424,7 @@ namespace GmmLib
                 pthread_mutex_unlock(&SingletonContextSyncMutex);
                 return GMM_SUCCESS;
             }
+            #endif //!GMM_LIB_DLL_MA
 
         #endif // _WIN32
 
@@ -554,9 +568,9 @@ namespace GmmLib
         {
             Override.pTextureCalc = pTextureCalc;
         }
-    #endif
+    #endif // (_DEBUG || _RELEASE_INTERNAL)
 
-    #endif
+    #endif // __GMM_KMD__
 
     GMM_CACHE_POLICY* GMM_STDCALL CreateCachePolicyCommon();
     GMM_TEXTURE_CALC* GMM_STDCALL CreateTextureCalc(PLATFORM Platform, bool Override);
@@ -589,9 +603,6 @@ namespace GmmLib
 typedef struct _GMM_ADAPTER_INFO_
 {
     Context             *pGmmLibContext;                    // Gmm UMD Lib Context which is process Singleton
-    int32_t             RefCount;                           // Ref Count for the number of Gmm UMD Lib process Singleton Context created per Process
-    GMM_MUTEX_HANDLE    SyncMutex;                          // SyncMutex to protect access of Gmm UMD Lib process Singleton Context 
-    ADAPTER_BDF         sBdf;                               // Adpater's Bus, Device and Function info for which Gmm UMD Lib process Singleton Context is created
     _GMM_ADAPTER_INFO_  *pNext;                             // Linked List Next pointer to point to the Next Adapter node in the List
 
 }GMM_ADAPTER_INFO;
@@ -612,26 +623,44 @@ typedef struct _GMM_ADAPTER_INFO_
                                                    // The Multi-Adapter Initialization is done dynamiclly using a Linked list Vector
                                                    // pHeadNode points to the root node of the linked list and registers the first
                                                    // adapter received from UMD.
+        // thread safe functions; these cannot be called within a LockMAContextSyncMutex block
+        GMM_ADAPTER_INFO *              GetAdapterNode(ADAPTER_BDF sBdf);   // Replacement for GetAdapterIndex, now get adapter node from the linked list
+
+        // Mutexes which protect the below thread unsafe functions
+        GMM_STATUS                      LockMAContextSyncMutex();
+        GMM_STATUS                      UnLockMAContextSyncMutex();
+
+        // thread unsafe functions; these must be protected with LockMAContextSyncMutex
+        GMM_ADAPTER_INFO *              GetAdapterNodeUnlocked(ADAPTER_BDF sBdf);
+        GMM_ADAPTER_INFO *              AddAdapterNode();
+        void                            RemoveAdapterNode(GMM_ADAPTER_INFO *pNode);
+
     public:
         //Constructors and destructors
         GmmMultiAdapterContext();
         ~GmmMultiAdapterContext();
         /* Function prototypes */
-        /* Fucntions that update MultiAdapterContext members*/
+        /* Functions that update MultiAdapterContext members*/
         uint32_t GMM_STDCALL            GetAdapterIndex(ADAPTER_BDF sBdf);
-        GMM_STATUS GMM_STDCALL          IntializeAdapterInfo(ADAPTER_BDF sBdf);
-        void GMM_STDCALL                ReleaseAdapterInfo(ADAPTER_BDF sBdf);
-        Context* GMM_STDCALL            GetAdapterLibContext(ADAPTER_BDF sBdf);
-        void GMM_STDCALL                SetAdapterLibContext(ADAPTER_BDF sBdf, Context* pGmmLibContext);
-        GMM_STATUS GMM_STDCALL          LockMAContextSyncMutex();
-        GMM_STATUS GMM_STDCALL          UnLockMAContextSyncMutex();
         uint32_t GMM_STDCALL            GetNumAdapters();
-        /* Fucntions that update AdapterInfo*/
-        int32_t GMM_STDCALL             IncrementRefCount(ADAPTER_BDF sBdf);
-        int32_t GMM_STDCALL             DecrementRefCount(ADAPTER_BDF sBdf);
-        GMM_STATUS GMM_STDCALL          LockSingletonContextSyncMutex(ADAPTER_BDF sBdf);
-        GMM_STATUS GMM_STDCALL          UnlockSingletonContextSyncMutex(ADAPTER_BDF sBdf);
-        void *GMM_STDCALL               GetAdapterNode(ADAPTER_BDF sBdf); // Replacemet for GetAdapterIndex, now get adapter info from a node in the linked list
+        /* Functions that update AdapterInfo*/
+        // thread safe functions; these cannot be called within a LockMAContextSyncMutex block
+#if LHDM
+        GMM_STATUS GMM_STDCALL          AddContext(const PLATFORM           Platform,
+                                                   const SKU_FEATURE_TABLE *pSkuTable,
+                                                   const WA_TABLE *         pWaTable,
+                                                   const GT_SYSTEM_INFO *   pGtSysInfo,
+                                                   ADAPTER_BDF              sBdf,
+                                                   const char *             DeviceRegistryPath);
+#else
+        GMM_STATUS GMM_STDCALL          AddContext(const PLATFORM Platform,
+                                                   const void *   pSkuTable,
+                                                   const void *   pWaTable,
+                                                   const void *   pGtSysInfo,
+                                                   ADAPTER_BDF    sBdf);
+#endif
+        GMM_STATUS GMM_STDCALL          RemoveContext(ADAPTER_BDF sBdf);
+        Context* GMM_STDCALL            GetAdapterLibContext(ADAPTER_BDF sBdf);
     }; // GmmMultiAdapterContext
 
 } //namespace
